#!/usr/bin/env ruby

require 'flexirecord'

# Copyright (c) 2007 FlexiGuided GmbH, Berlin
#
# Author: Jan Behrens
#
# Website: http://www.flexiguided.de/publications.flexirecord.en.html
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
# -----
#
# Demonstration module for FlexiRecord.

module FlexiRecordDemo

  # FlexiRecord::ConnectionPool used for all models in this module.
  ConnectionPool = FlexiRecord::BaseRecord.connection_pool = FlexiRecord::ConnectionPool.new(:engine => :postgresql, :db => 'moviedemo')

  # A Medium (demo class).
  #
  # CREATE TABLE "medium" ("number" int8 primary key );
  class Medium < FlexiRecord::BaseRecord
    self.table_name = 'medium'
    def after_select(records)
      super
      records.preload(:movies)
    end
    def save
      if self.number == :auto
        self.class.transaction self, :read_committed do
          self.class.db_execute("LOCK TABLE #{self.class.table} IN SHARE ROW EXCLUSIVE MODE")
          last_medium = Medium.select1('ORDER BY "number" DESC LIMIT 1')
          self.number = if last_medium
            last_medium.number + 1
          else
            1
          end
          return super
        end
      else
        return super
      end
    end
  end

  # A Movie which is stored on a Medium (demo class).
  #
  # CREATE TABLE "movie" ("id" serial8 primary key, "medium_number" int8            not null references "medium" ("number") on delete cascade on update cascade, "name" text );
  class Movie < FlexiRecord::BaseRecord
    self.table_name = 'movie'
    add_many_to_one_reference Medium, 'medium_', :medium, :movies
  end

  # A small demonstration program.
  def demo
    Medium.transaction do
      medium = FlexiRecordDemo::Medium.new(:number => :auto).save
      movie = FlexiRecordDemo::Movie.new(:medium => medium, :name => 'Naruto').save
    end
  end
  module_function :demo

end
